!==============================================================================
! communicate boundaries of mass matrix
!==============================================================================

subroutine comm_M
use parameters
use variables
implicit none
include 'mpif.h'

	integer :: mi_rec, mi_src
	integer :: send_tag, rec_tag, num
        integer :: status(MPI_STATUS_SIZE)

        real, dimension(0:ny_max,0:nz_max,0:lpd,0:lpd) :: Bx
        real, dimension(0:nx_max,0:nz_max,0:lpd,0:lpd) :: By
        real, dimension(0:nx_max,0:ny_max,0:lpd,0:lpd) :: Bz
	
!------------------------------------------------------------------------------

	!======================================================================
	! communication in x direction
	!======================================================================
	
	num=(ny+1)*(nz+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi-1)*py*px+(iy_multi-1)*px+ix_multi
        mi_src=(iz_multi-1)*py*px+(iy_multi-1)*px+ix_multi-2

        !write(*,*) ix_multi, mi_rec, mi_src

	if (ix_multi<px) then
		
                !write(99,*) 'mi_rec=', mi_rec
	
		call mpi_send(MM(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,10,mpi_comm_world,ierr)

        endif

        if (ix_multi>1) then

                !write(99,*) 'mi_src=', mi_src

		call mpi_recv(Bx(0:ny,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,10,mpi_comm_world,status,ierr)

                MM(0,0:ny,0:nz,0,0:lpd,0:lpd)=MM(0,0:ny,0:nz,0,0:lpd,0:lpd)+Bx(0:ny,0:nz,0:lpd,0:lpd)

                call mpi_send(MM(0,0:ny,0:nz,0,0:lpd,0:lpd),num,mpi_real,mi_src,11,mpi_comm_world,status,ierr)

	endif

        if (ix_multi<px) then

                call mpi_recv(MM(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,11,mpi_comm_world,status,ierr)

        endif
!if (1==0) then
        !======================================================================
	! communication in y direction
	!======================================================================
	
	num=(nx+1)*(nz+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi-1)*py*px+(iy_multi)*px+ix_multi-1
        mi_src=(iz_multi-1)*py*px+(iy_multi-2)*px+ix_multi-1

	if (iy_multi<py) then
			
		call mpi_send(MM(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,20,mpi_comm_world,ierr)

        endif

        if (iy_multi>1) then

		call mpi_recv(By(0:nx,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,20,mpi_comm_world,status,ierr)

                MM(0:nx,0,0:nz,0:lpd,0,0:lpd)=MM(0:nx,0,0:nz,0:lpd,0,0:lpd)+By(0:nx,0:nz,0:lpd,0:lpd)

                call mpi_send(MM(0:nx,0,0:nz,0:lpd,0,0:lpd),num,mpi_real,mi_src,21,mpi_comm_world,status,ierr)

	endif

        if (iy_multi<py) then

                call mpi_recv(MM(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,21,mpi_comm_world,status,ierr)

        endif

	
        !======================================================================
	! communication in z direction
	!======================================================================
	
	num=(nx+1)*(ny+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi)*py*px+(iy_multi-1)*px+ix_multi-1
        mi_src=(iz_multi-2)*py*px+(iy_multi-1)*px+ix_multi-1

	if (iz_multi<pz) then
			
		call mpi_send(MM(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,30,mpi_comm_world,ierr)

        endif

        if (iz_multi>1) then

		call mpi_recv(Bz(0:nx,0:ny,0:lpd,0:lpd),num,mpi_real,mi_src,30,mpi_comm_world,status,ierr)

                MM(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)=MM(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)+Bz(0:nx,0:ny,0:lpd,0:lpd)

                call mpi_send(MM(0:nx,0:ny,nz,0:lpd,0:lpd,lpd),num,mpi_real,mi_src,31,mpi_comm_world,status,ierr)

	endif

        if (iz_multi<pz) then

                call mpi_recv(MM(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,31,mpi_comm_world,status,ierr)

        endif
!endif
end subroutine comm_M


!==============================================================================
! communicate boundaries of weak form stresses
!==============================================================================

subroutine comm_S(field_x, field_y, field_z)
use parameters
use variables
implicit none
include 'mpif.h'

        integer :: i, j, k
	integer :: mi_rec, mi_src
	integer :: send_tag, rec_tag, num
        integer :: status(MPI_STATUS_SIZE)

        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd), intent(inout) :: field_x, field_y, field_z

        real, dimension(0:ny_max,0:nz_max,0:lpd,0:lpd) :: Bxx, Byx, Bzx
        real, dimension(0:nx_max,0:nz_max,0:lpd,0:lpd) :: Bxy, Byy, Bzy
        real, dimension(0:nx_max,0:ny_max,0:lpd,0:lpd) :: Bxz, Byz, Bzz
	

        !======================================================================
        ! communication within one processor
        !======================================================================

        do i=1,nx

           field_x(i,:,:,0,:,:)=field_x(i,:,:,0,:,:)+field_x(i-1,:,:,lpd,:,:)
           field_x(i-1,:,:,lpd,:,:)=field_x(i,:,:,0,:,:);

           field_y(i,:,:,0,:,:)=field_y(i,:,:,0,:,:)+field_y(i-1,:,:,lpd,:,:)
           field_y(i-1,:,:,lpd,:,:)=field_y(i,:,:,0,:,:);

           field_z(i,:,:,0,:,:)=field_z(i,:,:,0,:,:)+field_z(i-1,:,:,lpd,:,:)
           field_z(i-1,:,:,lpd,:,:)=field_z(i,:,:,0,:,:);

        enddo

        do j=1,ny

           field_x(:,j,:,:,0,:)=field_x(:,j,:,:,0,:)+field_x(:,j-1,:,:,lpd,:)
           field_x(:,j-1,:,:,lpd,:)=field_x(:,j,:,:,0,:)

           field_y(:,j,:,:,0,:)=field_y(:,j,:,:,0,:)+field_y(:,j-1,:,:,lpd,:)
           field_y(:,j-1,:,:,lpd,:)=field_y(:,j,:,:,0,:)

           field_z(:,j,:,:,0,:)=field_z(:,j,:,:,0,:)+field_z(:,j-1,:,:,lpd,:)
           field_z(:,j-1,:,:,lpd,:)=field_z(:,j,:,:,0,:)

        enddo

        do k=1,nz

           field_x(:,:,k,:,:,0)=field_x(:,:,k,:,:,0)+field_x(:,:,k-1,:,:,lpd)
           field_x(:,:,k-1,:,:,lpd)=field_x(:,:,k,:,:,0)

           field_y(:,:,k,:,:,0)=field_y(:,:,k,:,:,0)+field_y(:,:,k-1,:,:,lpd)
           field_y(:,:,k-1,:,:,lpd)=field_y(:,:,k,:,:,0)
           
           field_z(:,:,k,:,:,0)=field_z(:,:,k,:,:,0)+field_z(:,:,k-1,:,:,lpd)
           field_z(:,:,k-1,:,:,lpd)=field_z(:,:,k,:,:,0)

        enddo


        !======================================================================
        ! communication between processors
        !======================================================================

	!======================================================================
	! communication in x direction
	!======================================================================
	
	num=(ny+1)*(nz+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi-1)*py*px+(iy_multi-1)*px+ix_multi
        mi_src=(iz_multi-1)*py*px+(iy_multi-1)*px+ix_multi-2

	if (ix_multi<px) then
			
		call mpi_send(field_x(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,110,mpi_comm_world,ierr)
		call mpi_send(field_y(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,111,mpi_comm_world,ierr)
		call mpi_send(field_z(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,112,mpi_comm_world,ierr)

        endif

        if (ix_multi>1) then

		call mpi_recv(Bxx(0:ny,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,110,mpi_comm_world,status,ierr)
		call mpi_recv(Byx(0:ny,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,111,mpi_comm_world,status,ierr)
		call mpi_recv(Bzx(0:ny,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,112,mpi_comm_world,status,ierr)

                field_x(0,0:ny,0:nz,0,0:lpd,0:lpd)=field_x(0,0:ny,0:nz,0,0:lpd,0:lpd)+Bxx(0:ny,0:nz,0:lpd,0:lpd)
                field_y(0,0:ny,0:nz,0,0:lpd,0:lpd)=field_y(0,0:ny,0:nz,0,0:lpd,0:lpd)+Byx(0:ny,0:nz,0:lpd,0:lpd)
                field_z(0,0:ny,0:nz,0,0:lpd,0:lpd)=field_z(0,0:ny,0:nz,0,0:lpd,0:lpd)+Bzx(0:ny,0:nz,0:lpd,0:lpd)

                call mpi_send(field_x(0,0:ny,0:nz,0,0:lpd,0:lpd),num,mpi_real,mi_src,120,mpi_comm_world,status,ierr)
                call mpi_send(field_y(0,0:ny,0:nz,0,0:lpd,0:lpd),num,mpi_real,mi_src,121,mpi_comm_world,status,ierr)
                call mpi_send(field_z(0,0:ny,0:nz,0,0:lpd,0:lpd),num,mpi_real,mi_src,122,mpi_comm_world,status,ierr)

	endif

        if (ix_multi<px) then

                call mpi_recv(field_x(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,120,mpi_comm_world,status,ierr)
                call mpi_recv(field_y(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,121,mpi_comm_world,status,ierr)
                call mpi_recv(field_z(nx,0:ny,0:nz,lpd,0:lpd,0:lpd),num,mpi_real,mi_rec,122,mpi_comm_world,status,ierr)

        endif

        !======================================================================
	! communication in y direction
	!======================================================================

	num=(nx+1)*(nz+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi-1)*py*px+(iy_multi)*px+ix_multi-1
        mi_src=(iz_multi-1)*py*px+(iy_multi-2)*px+ix_multi-1

	if (iy_multi<py) then
			
		call mpi_send(field_x(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,210,mpi_comm_world,ierr)
		call mpi_send(field_y(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,211,mpi_comm_world,ierr)
		call mpi_send(field_z(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,212,mpi_comm_world,ierr)

        endif

        if (iy_multi>1) then

		call mpi_recv(Bxy(0:nx,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,210,mpi_comm_world,status,ierr)
		call mpi_recv(Byy(0:nx,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,211,mpi_comm_world,status,ierr)
		call mpi_recv(Bzy(0:nx,0:nz,0:lpd,0:lpd),num,mpi_real,mi_src,212,mpi_comm_world,status,ierr)

                field_x(0:nx,0,0:nz,0:lpd,0,0:lpd)=field_x(0:nx,0,0:nz,0:lpd,0,0:lpd)+Bxy(0:nx,0:nz,0:lpd,0:lpd)
                field_y(0:nx,0,0:nz,0:lpd,0,0:lpd)=field_y(0:nx,0,0:nz,0:lpd,0,0:lpd)+Byy(0:nx,0:nz,0:lpd,0:lpd)
                field_z(0:nx,0,0:nz,0:lpd,0,0:lpd)=field_z(0:nx,0,0:nz,0:lpd,0,0:lpd)+Bzy(0:nx,0:nz,0:lpd,0:lpd)

                call mpi_send(field_x(0:nx,0,0:nz,0:lpd,0,0:lpd),num,mpi_real,mi_src,220,mpi_comm_world,status,ierr)
                call mpi_send(field_y(0:nx,0,0:nz,0:lpd,0,0:lpd),num,mpi_real,mi_src,221,mpi_comm_world,status,ierr)
                call mpi_send(field_z(0:nx,0,0:nz,0:lpd,0,0:lpd),num,mpi_real,mi_src,222,mpi_comm_world,status,ierr)

	endif

        if (iy_multi<py) then

                call mpi_recv(field_x(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,220,mpi_comm_world,status,ierr)
                call mpi_recv(field_y(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,221,mpi_comm_world,status,ierr)
                call mpi_recv(field_z(0:nx,ny,0:nz,0:lpd,lpd,0:lpd),num,mpi_real,mi_rec,222,mpi_comm_world,status,ierr)

        endif

	
        !======================================================================
	! communication in z direction
	!======================================================================
	
	num=(nx+1)*(ny+1)*(lpd+1)*(lpd+1)
	
        ! send boundary to right processor, if there is one

	mi_rec=(iz_multi)*py*px+(iy_multi-1)*px+ix_multi-1
        mi_src=(iz_multi-2)*py*px+(iy_multi-1)*px+ix_multi-1

	if (iz_multi<pz) then
			
		call mpi_send(field_x(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,310,mpi_comm_world,ierr)
		call mpi_send(field_y(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,311,mpi_comm_world,ierr)
		call mpi_send(field_z(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,312,mpi_comm_world,ierr)

        endif

        if (iz_multi>1) then

		call mpi_recv(Bxz(0:nx,0:ny,0:lpd,0:lpd),num,mpi_real,mi_src,310,mpi_comm_world,status,ierr)
		call mpi_recv(Byz(0:nx,0:ny,0:lpd,0:lpd),num,mpi_real,mi_src,311,mpi_comm_world,status,ierr)
		call mpi_recv(Bzz(0:nx,0:ny,0:lpd,0:lpd),num,mpi_real,mi_src,312,mpi_comm_world,status,ierr)

                field_x(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)=field_x(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)+Bxz(0:nx,0:ny,0:lpd,0:lpd)
                field_y(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)=field_y(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)+Byz(0:nx,0:ny,0:lpd,0:lpd)
                field_z(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)=field_z(0:nx,0:ny,nz,0:lpd,0:lpd,lpd)+Bzz(0:nx,0:ny,0:lpd,0:lpd)

                call mpi_send(field_x(0:nx,0:ny,nz,0:lpd,0:lpd,lpd),num,mpi_real,mi_src,320,mpi_comm_world,status,ierr)
                call mpi_send(field_y(0:nx,0:ny,nz,0:lpd,0:lpd,lpd),num,mpi_real,mi_src,321,mpi_comm_world,status,ierr)
                call mpi_send(field_z(0:nx,0:ny,nz,0:lpd,0:lpd,lpd),num,mpi_real,mi_src,322,mpi_comm_world,status,ierr)

	endif

        if (iz_multi<pz) then

                call mpi_recv(field_x(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,320,mpi_comm_world,status,ierr)
                call mpi_recv(field_y(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,321,mpi_comm_world,status,ierr)
                call mpi_recv(field_z(0:nx,0:ny,0,0:lpd,0:lpd,0),num,mpi_real,mi_rec,322,mpi_comm_world,status,ierr)

        endif
		
end subroutine comm_S
