subroutine sec3d_init
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================

	character(len=20) :: dummy
	integer :: status(MPI_STATUS_SIZE)
	integer :: i, j, k, l, m, n
        real :: delta_x, delta_y, delta_z, alpha
        real :: sr
    
	!======================================================================
	
	call int2str(my_rank, dummy)
	
    	if (my_rank==0) then
    
	    write(*,*)'begin init '
    	    
	endif
	
	write(99,*)'begin init '
	write(99,*) '----------------------------------'

    
        !======================================================================
        ! elastic parameters and memory variables
        !======================================================================

        kappa=lambda+2*mu/3

        if (is_diss==1) then

           mu_tau=(1+tau)*mu

           Mxx(:,:,:,:,:,:,:)=0.0
           Myy(:,:,:,:,:,:,:)=0.0
           Mzz(:,:,:,:,:,:,:)=0.0

           Mxy(:,:,:,:,:,:,:)=0.0
           Mxz(:,:,:,:,:,:,:)=0.0
           Myz(:,:,:,:,:,:,:)=0.0

        endif

        !======================================================================
        ! adjoint calculations
        !======================================================================

        saving_vector(:)=0.0
		
	!======================================================================
	! determine collocation points (knots) and integration weights
	!======================================================================
	
	if (lpd==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
		w(0)=1/3
		w(1)=4/3
		w(2)=1/3
		
	elseif (lpd==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
		w(0)=0.1666666666666
		w(1)=0.8333333333333
		w(2)=0.8333333333333
		w(3)=0.1666666666666
		
	elseif (lpd==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
		w(0)=0.1000000000000
		w(1)=0.5444444444444
		w(2)=0.7111111111111
		w(3)=0.5444444444444
		w(4)=0.1000000000000
		
	elseif (lpd==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
		w(0)=0.0666666666666667
		w(1)=0.3784749562978470
		w(2)=0.5548583770354862
		w(3)=0.5548583770354862
		w(4)=0.3784749562978470
		w(5)=0.0666666666666667
		
	elseif (lpd==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
		w(0)=0.0476190476190476
		w(1)=0.2768260473615659
		w(2)=0.4317453812098627
		w(3)=0.4876190476190476
		w(4)=0.4317453812098627
		w(5)=0.2768260473615659
		w(6)=0.0476190476190476
		
	elseif (lpd==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
		w(0)=0.0357142857142857
		w(1)=0.2107042271435061
		w(2)=0.3411226924835044
		w(3)=0.4124587946587038
		w(4)=0.4124587946587038
		w(5)=0.3411226924835044
		w(6)=0.2107042271435061
		w(7)=0.0357142857142857
		
	endif

       
	!======================================================================
	! initialisation of LAGRANGE polynomial derivatives
	!======================================================================
	
	do i=0,lpd
		do j=0,lpd
			call dlgll(lpd,i,j,dl(i,j))
		enddo
	enddo
	
	!======================================================================
	! initialisation of the space matrices
	!======================================================================
	
	dx=(xmax-xmin)/(nx+1)		! width of one element in x direction
	dy=(ymax-ymin)/(ny+1)		! width of one element in y direction
	dz=(zmax-zmin)/(nz+1)		! width of one element in z direction
	
	write(99,*) 'width of one element in x direction: ', dx
	write(99,*) 'width of one element in y direction: ', dy
	write(99,*) 'width of one element in z direction: ', dz

	write(99,*)ofd
		
! don t understand why ofd is not recognised herr
	open(unit=101,file='../DATA/COORDINATES/xco_'//dummy(1:len_trim(dummy)),status='unknown')
	open(unit=102,file='../DATA/COORDINATES/yco_'//dummy(1:len_trim(dummy)),status='unknown')
	open(unit=103,file='../DATA/COORDINATES/zco_'//dummy(1:len_trim(dummy)),status='unknown')
!	open(unit=101,file=ofd(1:len_trim(ofd))//'COORDINATES/xco_'//dummy(1:len_trim(dummy)),status='unknown')
!	open(unit=102,file=ofd(1:len_trim(ofd))//'COORDINATES/yco_'//dummy(1:len_trim(dummy)),status='unknown')
!	open(unit=103,file=ofd(1:len_trim(ofd))//'COORDINATES/zco_'//dummy(1:len_trim(dummy)),status='unknown')
	
	do i=0,nx
		do n=0,lpd
			x(i,n)=xmin+i*dx+0.5*(1+knots(n))*dx
			write(101,*) x(i,n)
		enddo
	enddo

	do j=0,ny
		do n=0,lpd
			y(j,n)=ymin+j*dy+0.5*(1+knots(n))*dy
			write(102,*) y(j,n)
		enddo
	enddo
	
	do k=0,nz
		do n=0,lpd
			z(k,n)=zmax-k*dz-0.5*(1+knots(n))*dz
			write(103,*) z(k,n)
		enddo
	enddo
	
	close(unit=101)
	close(unit=102)
	close(unit=103)

	!======================================================================
	! initialisation of the JACOBIAN
	!======================================================================
	
	Jac=dx*dy*dz/8

        !======================================================================
        ! initialisation of the mass matrix
        !======================================================================

        ! local mass matrix

        do i=0,nx
           do j=0,ny
              do k=0,nz
                 do l=0,lpd
                    do m=0,lpd
                       do n=0,lpd

                          MM(i,j,k,l,m,n)=w(l)*w(m)*w(n)*Jac/rhoinv(i,j,k,l,m,n)

                       enddo
                    enddo
                 enddo
              enddo
           enddo
        enddo

       
        ! interchange of elements -> global mass matrix

        do i=1,nx
           MM(i,:,:,0,:,:)=MM(i,:,:,0,:,:)+MM(i-1,:,:,lpd,:,:)
           MM(i-1,:,:,lpd,:,:)=MM(i,:,:,0,:,:)
        enddo

        do j=1,ny
           MM(:,j,:,:,0,:)=MM(:,j,:,:,0,:)+MM(:,j-1,:,:,lpd,:)
           MM(:,j-1,:,:,lpd,:)=MM(:,j,:,:,0,:)
        enddo

        do k=1,nz
           MM(:,:,k,:,:,0)=MM(:,:,k,:,:,0)+MM(:,:,k-1,:,:,lpd)
           MM(:,:,k-1,:,:,lpd)=MM(:,:,k,:,:,0)
        enddo

        ! communication (globalisation)

        call comm_M


        !=======================================================================
        ! determine boxwise receiver locations
        !=======================================================================

        nr=0

        write(99,*) '---------------------------------------------'
        write(99,*) 'receiver locations in transformed unit system'
        write(99,*) '---------------------------------------------'

        do n=1,nr_global

           if ((recloc_global(1,n)<=xmax) .and. (recloc_global(1,n)>xmin) .and. &
               (recloc_global(2,n)<=ymax) .and. (recloc_global(2,n)>ymin) .and. &
               (zmax_global-recloc_global(3,n)<=zmax) .and. (zmax_global-recloc_global(3,n)>zmin)) then

              write(99,*) recloc_global(1,n), recloc_global(2,n), zmax_global-recloc_global(3,n)

               nr=nr+1

               do i=0,nx
                  do j=0,ny
                     do k=0,nz

                        if ((recloc_global(1,n)<=x(i,lpd)) .and. (recloc_global(1,n)>x(i,0)) .and. &
                            (recloc_global(2,n)<=y(j,lpd)) .and. (recloc_global(2,n)>y(j,0)) .and. &
                            (zmax_global-recloc_global(3,n)<=z(k,0)) .and. (zmax_global-recloc_global(3,n)>z(k,lpd))) then
                           
                           ! receiver coordinates in unit system

                           recloc_std(1,nr)=2*((recloc_global(1,n)-xmin)/dx-i)-1
                           recloc_std(2,nr)=2*((recloc_global(2,n)-ymin)/dy-j)-1
                           recloc_std(3,nr)=-2*((zmax_global-recloc_global(3,n)-zmax)/dz+k)-1

                           recloc(1,nr)=recloc_global(1,n)
                           recloc(2,nr)=recloc_global(2,n)
                           recloc(3,nr)=recloc_global(3,n)

                           rx(nr)=i
                           ry(nr)=j
                           rz(nr)=k
                           
                           write(99,*) 'element: (', i,',',j,',',k,'), coordinates: (', & 
                                recloc_std(1,nr),',',recloc_std(2,nr),',',recloc_std(3,nr),')'

                        endif

                     enddo
                  enddo
               enddo

            endif

        enddo

        write(99,*) 'number of receivers: ', nr

        seismogram_x(:,:)=0.0
        seismogram_y(:,:)=0.0
        seismogram_z(:,:)=0.0
	
     	!=========================================================================
	! point source location
	!=========================================================================
	
	if (is_source==1) then
		
		delta_x=dx
		delta_y=dy
		delta_z=dz
		
		! search for source location in x direction
		
		do i=0,nx
			do n=0,lpd
			
				if (abs(x(i,n)-xxs)<delta_x) then
					
					delta_x=abs(x(i,n)-xxs)
					isx=i
					isx_n=n
					
				endif
				
			enddo
		enddo
		
		! search for source location in y direction
		
		do j=0,ny
			do n=0,lpd
			
				if (abs(y(j,n)-yys)<delta_y) then
					
					delta_y=abs(y(j,n)-yys)
					isy=j
					isy_n=n
					
				endif
				
			enddo
		enddo
		
		! search for source location in z direction
		
		do k=0,nz
			do n=0,lpd
			
				if (abs(zmax_global-z(k,n)-zzs)<delta_z) then
					
					delta_z=abs(zmax_global-z(k,n)-zzs)
					isz=k
					isz_n=n
					
				endif
				
			enddo
		enddo
		
		write(99,*)'Source centered in grid point: (',isx,', ',isx_n,'), (',isy,', ',isy_n,'), (',isz,', ',isz_n,')'
		write(99,*)'Source coordinates: ',x(isx,isx_n),y(isy,isy_n),z(isz,isz_n)
		write(99,*)'Source mislocation: ',xxs-x(isx,isx_n),yys-y(isy,isy_n),zmax_global-zzs-z(isz,isz_n)
		
	endif

        !======================================================================
        ! initialise source field
        !======================================================================

        sf_dx=0.0
        sf_dy=0.0
        sf_dz=0.0

        if (isx_n<2) then
                isx_n=2
        elseif (isx_n>lpd-2) then
                isx_n=lpd-2
        endif

        if (isy_n<2) then
                isy_n=2
        elseif (isy_n>lpd-2) then
                isy_n=lpd-2
        endif

        if (isz_n<2) then
                isz_n=2
        elseif (isz_n>lpd-2) then
                isz_n=lpd-2
        endif

        !- derivative in x direction

        if ((x(isx,isx_n)>xxs) .and. (is_source==1)) then

                sf_dx(isx,isy,isz,isx_n,isy_n,isz_n)=-1.0/(x(isx,isx_n)-x(isx,isx_n-1))
                sf_dx(isx,isy,isz,isx_n-1,isy_n,isz_n)=1.0/(x(isx,isx_n)-x(isx,isx_n-1))

        elseif ((x(isx,isx_n)<=xxs) .and. (is_source==1)) then

                sf_dx(isx,isy,isz,isx_n,isy_n,isz_n)=1.0/(x(isx,isx_n+1)-x(isx,isx_n))
                sf_dx(isx,isy,isz,isx_n+1,isy_n,isz_n)=-1.0/(x(isx,isx_n+1)-x(isx,isx_n))

        endif

        !- derivative in y direction

        if ((y(isy,isy_n)>yys) .and. (is_source==1)) then

                sf_dy(isx,isy,isz,isx_n,isy_n,isz_n)=-1.0/(y(isy,isy_n)-y(isy,isy_n-1))
                sf_dy(isx,isy,isz,isx_n,isy_n-1,isz_n)=1.0/(y(isy,isy_n)-y(isy,isy_n-1))

        elseif ((y(isy,isy_n)<=yys) .and. (is_source==1)) then

                sf_dy(isx,isy,isz,isx_n,isy_n,isz_n)=1.0/(y(isy,isy_n+1)-y(isy,isy_n))
                sf_dy(isx,isy,isz,isx_n,isy_n+1,isz_n)=-1.0/(y(isy,isy_n+1)-y(isy,isy_n))

        endif

        !- derivative in z direction

        if ((z(isz,isz_n)>(zmax_global-zzs)) .and. (is_source==1)) then

                sf_dz(isx,isy,isz,isx_n,isy_n,isz_n)=-1.0/(z(isz,isz_n)-z(isz,isz_n+1))
                sf_dz(isx,isy,isz,isx_n,isy_n,isz_n+1)=1.0/(z(isz,isz_n)-z(isz,isz_n+1))

        elseif ((z(isz,isz_n)<=(zmax_global-zzs)) .and. (is_source==1)) then

                sf_dz(isx,isy,isz,isx_n,isy_n,isz_n)=1.0/(z(isz,isz_n-1)-z(isz,isz_n))
                sf_dz(isx,isy,isz,isx_n,isy_n,isz_n-1)=-1.0/(z(isz,isz_n-1)-z(isz,isz_n))

        endif
  
        !======================================================================
        ! PML damping profiles
        !======================================================================


        alpha=1.2

        prof_x=0.0
        prof_y=0.0
        prof_z=0.0
        
        !- upper z-boundary

!        if (iz_multi==pz) then

!           do k=0,pml-1
!              do n=0,lpd

!                 delta_z=(zmax-pml*dz-z(k,n))/(pml*dz)
!                 prof_z(0:nx,0:ny,k,0:lpd,0:lpd,n)=alpha*delta_z*delta_z

!              enddo
!           enddo

!        endif

        !- lower z-boundary

        if (iz_multi==1) then

           do k=(nz-pml+1),nz
              do n=0,lpd

                 delta_z=(zmin+pml*dz-z(k,n))/(pml*dz)
                 prof_z(0:nx,0:ny,k,0:lpd,0:lpd,n)=alpha*delta_z*delta_z

              enddo
           enddo

        endif

        !- left x-boundary
                 
        if (ix_multi==1) then

           do i=0,pml-1
              do n=0,lpd

                 delta_x=(xmin+pml*dx-x(i,n))/(pml*dx)
                 prof_x(i,0:ny,0:nz,n,0:lpd,0:lpd)=alpha*delta_x*delta_x

              enddo
           enddo

        endif

        !- right x-boundary

        if (ix_multi==px) then

           do i=(nx-pml+1),nx
              do n=0,lpd

                 delta_x=(xmax-dx*pml-x(i,n))/(pml*dx)
                 prof_x(i,0:ny,0:nz,n,0:lpd,0:lpd)=alpha*delta_x*delta_x

              enddo
           enddo

        endif


        !- left y-boundary
                 
        if (iy_multi==1) then

           do i=0,pml-1
              do n=0,lpd

                 delta_y=(ymin+pml*dy-y(i,n))/(pml*dy)
                 prof_y(0:nx,i,0:nz,0:lpd,n,0:lpd)=alpha*delta_y*delta_y

              enddo
           enddo

        endif

        !- right y-boundary

        if (iy_multi==py) then

           do i=(ny-pml+1),ny
              do n=0,lpd

                 delta_y=(ymax-dy*pml-y(i,n))/(pml*dy)
                 prof_y(0:nx,i,0:nz,0:lpd,n,0:lpd)=alpha*delta_y*delta_y

              enddo
           enddo

        endif

        !- normalize the corners

!        prof_x=0.0
!        prof_y=0.0

        prof=prof_x+prof_y+prof_z

        do i=0,nx
           do j=0,ny
              do k=0,nz
                 do l=0,lpd
                    do m=0,lpd
                       do n=0,lpd
                          if (prof(i,j,k,l,m,n)>alpha) then
                             prof(i,j,k,l,m,n)=alpha
                          endif
                       enddo
                    enddo
                 enddo
              enddo
           enddo
        enddo

        taper=exp(-0.1*(prof*prof))

	!======================================================================
	! clean up
	!======================================================================

	if (my_rank==0) then

		write(*,*) 'end init'
	
	endif
	
	write(99,*) 'end init'
	write(99,*) '----------------------------------------'


end subroutine sec3d_init


real function sinc(xi)
implicit none

   real, intent(in) :: xi
   
   if (xi==0) then

        sinc=1.0

   else

        sinc=sin(xi)/xi

   endif

   return

end function sinc
