!*****************************************************************************
!***************** parameters and global variables ***************************
!*****************************************************************************

!=============================================================================
! Predefined parameters
!=============================================================================

module parameters
implicit none

	integer, parameter :: nx_max=15		! maximum number of elements in x direction
	integer, parameter :: ny_max=15		! maximum number of elements in y direction
	integer, parameter :: nz_max=15		! maximum number of elements in z direction
	
	integer, parameter :: lpd=4		! LAGRANGE polynomial degree
        integer, parameter :: pml=3             ! width of the PML region in elements
	
	integer, parameter :: maxnt=15000	! maximum number of time steps
	integer, parameter :: maxnr=50         ! maximum number of receivers
	
        integer, parameter :: nrdiss=2          ! number of relaxation mechanisms

	real, parameter :: pi=3.1415926535898

end module parameters
    
!==============================================================================
! global variables
!==============================================================================

module variables
use parameters
implicit none

	!======================================================================
	! model dimensions
	!======================================================================

	integer :: nx, ny, nz			! number of elements in the respective coordinate directions
	integer :: ix_multi, iy_multi, iz_multi	! multi index of the elements

	integer :: px, py, pz			! number of processors in the respective coordinate directions
	integer :: index_x, index_y, index_z
	integer :: p, my_rank, ierr, nabs

	!======================================================================
	! physical model parameters
	!======================================================================

        real, allocatable, dimension(:,:,:,:,:,:) :: rhoinv, mu, lambda, kappa, mu_tau, A, B, C, tau
        real, allocatable, dimension(:,:,:,:,:,:,:) :: tau_sp
	
	!======================================================================
	! displacement fields, strain fields, mass matrix and memory variables
	!======================================================================
	
        real, allocatable, dimension(:,:,:,:,:,:) :: ux, uy, uz, vx, vy, vz, sx, sy, sz
	real, allocatable, dimension(:,:,:,:,:,:) :: rotvx, rotvy, rotvz
        real, allocatable, dimension(:,:,:,:,:,:) :: exx, eyy, ezz, exy, eyx, exz, ezx, eyz, ezy

        real, allocatable, dimension(:,:,:,:,:,:) :: MM
        
        real, allocatable, dimension(:,:,:,:,:,:,:) :: Mxx, Myy, Mzz, Mxy, Mxz, Myz
	
        !======================================================================
        ! APML variables
        !======================================================================

        real, allocatable, dimension(:,:,:,:,:,:) :: prof_x, prof_y, prof_z, prof, taper

	!======================================================================
	! strong form stress fields
	!======================================================================
	
        real, allocatable, dimension(:,:,:,:,:,:) :: sxx, syy, szz, sxy, syx, sxz, szx, szy, syz
        real, allocatable, dimension(:,:,:,:,:,:) :: sxx_pml, syy_pml, szz_pml, sxy_pml, syx_pml, sxz_pml, szx_pml, szy_pml, syz_pml

        real, allocatable,  dimension(:,:,:,:,:,:) :: dxux, dyux, dzux, dxuy, dyuy, dzuy, dxuz, dyuz, dzuz
	
	!======================================================================
	! geometrical parameters
	!======================================================================
	
	real, dimension(0:7) :: knots, w
	real, dimension(0:7,0:7) :: dl
	
	real :: dx, dy, dz
	real :: xmin,xmax,ymin,ymax,zmin,zmax
	real :: xmin_global, xmax_global, ymin_global, ymax_global, zmin_global, zmax_global
	
        real, allocatable, dimension(:,:) :: x, y, z
	
        real :: Jac
	
	!======================================================================
	! time parameters
	!======================================================================
	
	integer :: it, nt
	real :: dt

	!======================================================================
	! Source Variables
	!======================================================================

	integer :: source_type
	integer :: isx, isy, isz, isx_n, isy_n, isz_n
!	real :: dip,rake,strike
	real :: xxs, yys, zzs
	real :: is_source, source_processor
	real, dimension(1:maxnt) :: so

        real, allocatable, dimension(:,:,:,:,:,:) :: source_field, sf_dx, sf_dy, sf_dz

        real :: MOMxx, MOMyy, MOMzz, MOMxy, MOMxz, MOMyz
	
	!======================================================================
	! receiver variables
	!======================================================================
	
        integer :: nr_global, nr                   			                        	! number of receivers (first line of recfile)
	real :: recloc_global(1:3,1:maxnr), recloc(1:3,1:maxnr), recloc_std(1:3,1:maxnr)     		! receiver location x, y, depth
        integer :: rx(1:maxnr), ry(1:maxnr), rz(1:maxnr)

        !======================================================================
        ! output
        !======================================================================

        real :: seismogram_x(1:maxnr,1:maxnt)
        real :: seismogram_y(1:maxnr,1:maxnt)
        real :: seismogram_z(1:maxnr,1:maxnt)
	
	real :: seismogram_rot_x(1:maxnr,1:maxnt)
        real :: seismogram_rot_y(1:maxnr,1:maxnt)
        real :: seismogram_rot_z(1:maxnr,1:maxnt)
	
	integer :: ssamp, iplanex, iplaney, iplanez
        character(len=100) :: ofd                                              ! output file directory
	
	!======================================================================
	! other variables
	!======================================================================
	
        real :: E_kin, E_kin_old
	integer :: is_aniso, is_diss, is_formatted

        !======================================================================
        ! adjoint computations
        !======================================================================

        integer :: samp_ad                                                    ! saving ratio for dynamic forward fields
        integer :: saving_vector(1:maxnt)                                     ! index for field storage
        character(len=50) :: fn_ad                                            ! directory for storing forward fields

end module variables

