function pq=z_Q(p)

%=========================================================================
% z_Q convolves a Seismogram with a filter that simulates dissipation
%=========================================================================

pq=p;

%=========================================================================
% input
%=========================================================================

c_0=4751.65;
x=1000e3;

tau=0.3292e-3;
tau_s1=9.3236;
tau_s2=1.0311;

T=p.nt*p.dt;
k=-p.nt:p.nt;
nu=k/T;
w=2*pi*nu;

i=sqrt(-1);

spectra=zeros(p.nr,2*p.nt+1);

%=========================================================================
% make complex c
%=========================================================================

c=(i*w*tau*tau_s1)./(1+i*w*tau_s1)+(i*w*tau*tau_s2)./(1+i*w*tau_s2);
c=0.5*c_0^2*c;
c=c_0^2+c;
%c=c./(i*(w+eps));

c=sqrt(c);

%=========================================================================
% make filter factor
%=========================================================================

F=exp(i*w*x/c_0).*((exp(-i*w*x./c)));

%=========================================================================
% compute FOURIER transforms
%=========================================================================

for n=1:p.nr
    
    for k=1:2*p.nt+1
    
        spectra(n,k)=p.dt*sum(pq.seismograms(n,1:p.nt).*exp(-2*pi*i*[0:(p.nt-1)]*(k-p.nt-1)/p.nt));
        
    end
    
    spectra(n,1:200)=0.0;
    spectra(n,(end-200):end)=0.0;

    spectra(n,:)=F.*spectra(n,:);
    
    %spectra(n,:)=F(:);
    
end

%plot(abs(spectra(1,:)));

%spectra(n,:)=F.*spectra(n,:);

%=========================================================================
% invert FOURIER transform
%=========================================================================

for m=1:p.nr
    
    for n=1:p.nt
        
        pq.seismograms(m,n)=(1/(p.dt*p.nt))*sum(spectra(m,:).*exp(2*pi*i*(n-1)*[-p.nt:p.nt]/p.nt));
        %pq.seismograms(m,n)=(1/(p.dt*p.nt))*sum(F.*exp(2*pi*i*(n-1)*[-p.nt:p.nt]/p.nt));
    end
    
end

