%function p=z_read_seismograms(array,spacing)

addpath /l1/heiner/Bin


close all
clear


% function p=z_read_seismograms(array,spacing)
%
% reads seismograms out of the output files and plots them
%
% INPUT:
%
% array:    array of the indeces of the seismograms that will be plotted,
%           enter [] if all seismograms are to be plotted
%
% spacing:  spacing of seismograms in the plot as multiple of the maximum
%           amplitude. A reasonable value is 2.0 .
%
% OUTPUT:   a seismogram structure
%
% last modified: 3 November, 2006


% Simplue GUI with defaults
dlg_title = 'Parameters for plotting';
prompt = {'Number of processors','Corner frequency (Hz)','Filter order', 'Normalization?','Seismogram spacing','Source (x) ','Source (y)','Sort for distance?'};
num_lines=1;
def ={'64','1','2','1','1','0','0','1'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);
np= str2num(char(answer(1)));
fup=str2num(char(answer(2)));
ord=str2num(char(answer(3)));
inorm=str2num(char(answer(4)));
spacing=str2num(char(answer(5)));
sx=str2num(char(answer(6)));
sy=str2num(char(answer(7)));
isort=str2num(char(answer(8)));

% End GUI


for ic=1:3,

    if ic==1, component='x';end
    if ic==2, component='y';end
    if ic==3, component='z';end


    % loop over all possible processors

    ir_global=0;

    for i=1:np,


        processor=sprintf('%i',i)

        filename=['../DATA/OUTPUT/seismogram_' component processor];
        %filename=['../DATA/OUTPUT/seismogram_rot_' component processor];

        fid=fopen(filename,'r');

        if fid>0,


            fgets(fid);
            fscanf(fid,'%c',4);
            p.nr=fscanf(fid,'%d',1);
            fscanf(fid,'%c',5);
            p.nt=fscanf(fid,'%d',1);
            fscanf(fid,'%c',5);
            p.dt=fscanf(fid,'%g',1);

            p.t0=0.0;

            for ir=1:p.nr

                ir_global=ir_global+1;
                fscanf(fid,'%c',4);
                p.r_x(ir_global)=fscanf(fid,'%g',1)
                fscanf(fid,'%c',4);
                p.r_y(ir_global)=fscanf(fid,'%g',1)
                fscanf(fid,'%c',4);
                p.r_z(ir_global)=fscanf(fid,'%g',1)
                p.r_dist(ir_global)=sqrt((p.r_x(ir_global)-sx)^2+(p.r_y(ir_global)-sy)^2);
                
                for it=1:p.nt
                    p.seismograms(ir_global,it)=fscanf(fid,'%g',1);
                end
                p.seismograms(ir_global,:)=lowpass(p.seismograms(ir_global,:),p.dt,fup,ord);
                if inorm==1,p.seismograms(ir_global,:)=p.seismograms(ir_global,:)/max(p.seismograms(ir_global,:));end

            end

        end

    end

    p.nr=ir_global;

    if isort==1,
        [junk ii]=sort(p.r_dist);
    end

    disp(sprintf(' Found %i seismograms, now plotting ... ',p.nr));


    % plot seismograms

    m=max(max(abs(p.seismograms)));
    t=0:p.dt:((p.nt-1)*p.dt);

    figure
    hold on

        for ir=1:p.nr
            plot(t,ir*spacing*m+p.seismograms(ii(ir),:),'b');
            text(1.05*t(end),ir*spacing*m,num2str(ii(ir)));
            %text(-0.17*t(end),ir*spacing*m+0.1*m,num2str(p.r_x(ii(ir))/1000));
            %text(-0.09*t(end),ir*spacing*m+0.1*m,num2str(p.r_y(ii(ir))/1000));
            %text(-0.01*t(end),ir*spacing*m+0.1*m,num2str(p.r_z(ii(ir))/1000));
        end

        %axis([0.*t(end) 1.2*t(end) 0 p.nr*spacing*m+2*m]);
        axis([0.*t(end) 1.2*t(end) 0 Inf]);      
        grid on

    

    text(-0.17*t(end), p.nr*spacing*m+1.1*m,'x');
    text(-0.09*t(end), p.nr*spacing*m+1.1*m,'y');
    text(-0.01*t(end), p.nr*spacing*m+1.1*m,'z [km]');

    xlabel('time [s]');

    if (strcmp(component,'x'))

        ylabel('x velocity [m/s]');

    elseif (strcmp(component,'y'))

        ylabel('y velocity [m/s]');

    elseif (strcmp(component,'z'))

        ylabel('z velocity[m/s]');

    end


end