function z_plot_seismograms(p,array,spacing)

% function z_plot_seismograms(p,array,spacing)
%
% plots seismograms from a seismogram structure
%
% INPUT:    
%
% p:        seismogram structure
% array:    array of the indeces of the seismograms that will be plotted,
%           enter [] if all seismograms are to be plotted
% spacing:  spacing of seismograms in the plot as multiple of the maximum
%           amplitude. A reasonable value is 2.0 .
%
% OUTPUT:   none
%
% last modified: 3 November, 2006

m=max(max(abs(p.seismograms)));
t=p.t0:p.dt:((p.nt-1)*p.dt)+p.t0;

figure
hold on


if (length(array)==0)

    for ir=1:p.nr
        plot(t,ir*spacing*m+p.seismograms(ir,:),'b');
        text(1.05*t(end),ir*spacing*m,num2str(ir));
        text(-0.17*t(end),ir*spacing*m+0.1*m,num2str(p.r_x(ir)/1000));
        text(-0.09*t(end),ir*spacing*m+0.1*m,num2str(p.r_y(ir)/1000));
	text(-0.01*t(end),ir*spacing*m+0.1*m,num2str(p.r_z(ir)/1000));
    end

    axis([-0.2*t(end) 1.1*t(end) 0 p.nr*spacing*m+2*m]);
    grid on
    
else
    
    for ir=1:length(array)
        plot(t,ir*spacing*m+p.seismograms(array(ir),:),'b');
        text(1.05*t(end)+5,ir*spacing*m,num2str(ir));
        text(-0.17*t(end),ir*spacing*m+0.1*m,num2str(p.r_x(array(ir))/1000));
        text(-0.09*t(end),ir*spacing*m+0.1*m,num2str(p.r_y(array(ir))/1000));
	text(-0.01*t(end),ir*spacing*m+0.1*m,num2str(p.r_z(array(ir))/1000));
    end

    axis([-0.2*t(end) 1.1*t(end) 0 p.nr*spacing*m+2*m]);
    grid on
    
end

text(-0.17*t(end), p.nr*spacing*m+1.1*m,'x');
text(-0.09*t(end), p.nr*spacing*m+1.1*m,'y');
text(-0.01*t(end), p.nr*spacing*m+1.1*m,'z [km]');

xlabel('time [s]');
