!=============================================================================
! model generation for ses3d
!=============================================================================

program generate_models
implicit none
include 'mpif.h'

	!======================================================================
	! local variables 
	!======================================================================

        !- parameters and indices ---------------------------------------------

	character(len=20) :: junk

	integer :: i, j, k, l, m, n 
	integer :: mi, is_aniso, is_formatted
	integer, allocatable, dimension(:,:) :: min_index, max_index		! minimum and maximum index boundaries
	integer :: ierr, p, my_rank

        real :: knots(0:7)
        real :: pi

        !- global model setup -------------------------------------------------

	integer :: model_type, status
	integer :: nx, ny, nz, px, py, pz, lpd
	integer :: bnx, bny, bnz

	real :: z_max, z_min, y_max, y_min, x_max, x_min, dz, dy, dx

        !- local model setup --------------------------------------------------

        integer :: nx_loc, ny_loc, nz_loc
        
        real :: x_max_loc, y_max_loc, z_max_loc
        real :: x_min_loc, y_min_loc, z_min_loc

        real, allocatable, dimension(:,:) :: x, y, z

        !- physical model -----------------------------------------------------
        
	real :: rho_homogen, mu_homogen, lambda_homogen, A_homogen, B_homogen, C_homogen
	real, allocatable, dimension(:,:,:,:,:,:) :: rhoinv, mu, lambda, A, B, C
        real, allocatable, dimension(:,:) :: cp1d, cs1d, rho1d, r
	
	real, dimension(1:139) :: ak135_r, ak135_cp, ak135_cs, ak135_rho
	real :: ratio
		
        pi=3.14159265358979

	!======================================================================
	! initialise mpi and open logfile
	!======================================================================

	call MPI_Init(ierr)
	call MPI_Comm_size(MPI_COMM_WORLD,p,ierr)
	call MPI_Comm_rank(MPI_COMM_WORLD,my_rank,ierr)

        call int2str(my_rank,junk)
	open(unit=99,file='logfile'//junk(1:len_trim(junk)),action='write')

	!======================================================================
	! process 0 reads input and broadcasts it to the other processes
	!======================================================================

	if (my_rank==0) then

		open(unit=10,file='model_parameters',action='READ')
	
		read(10,*) junk
		
		read(10,*) nx
		write(99,*) 'total number of elements in x-direction:', nx
		read(10,*) ny
		write(99,*) 'total number of elements in y-direction:', ny
		read(10,*) nz
		write(99,*) 'total number of elements in z-direction:', nz
		read(10,*) lpd
		write(99,*) 'LAGRANGE polynomial degree:', lpd
		read(10,*) x_min
		write(99,*) 'global minimum x value:', x_min
		read(10,*) x_max
		write(99,*) 'global maximum x value:', x_max
		read(10,*) y_min
		write(99,*) 'global minimum y value:', y_min
		read(10,*) y_max
		write(99,*) 'global maximum y value:', y_max
		read(10,*) z_min
		write(99,*) 'global minimum z value:', z_min
		read(10,*) z_max
		write(99,*) 'global maximum z value:', z_max
	
		read(10,*) junk
		read(10,*) model_type
		write(99,*) 'model type:', model_type
		read(10,*) mu_homogen
		write(99,*) 'homogeneous mu:', mu_homogen
		read(10,*) lambda_homogen
		write(99,*) 'homogeneous lambda:', lambda_homogen
		read(10,*) rho_homogen
		write(99,*) 'homogeneous rho:', rho_homogen
		read(10,*) A_homogen
		write(99,*) 'homogeneous A:', A_homogen
		read(10,*) B_homogen
		write(99,*) 'homogeneous B:', B_homogen
		read(10,*) C_homogen
		write(99,*) 'homogeneous C:', C_homogen
		read(10,*) is_aniso
		write(99,*) 'is anisotropic:', is_aniso
		read(10,*) is_formatted
		write(99,*) 'formatted output:', is_formatted
	
		read(10,*) junk
		read(10,*) px
		write(99,*) 'number of processors in x direction:', px
		read(10,*) py
		write(99,*) 'number of processors in y direction:', py
		read(10,*) pz
		write(99,*) 'number of processors in z direction:', pz
	
		close(unit=10)
		
	endif
	
	call mpi_bcast(nx,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(ny,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(nz,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(lpd,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(x_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(x_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(y_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(y_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(z_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(z_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(model_type,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(mu_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(lambda_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(rho_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(A_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(B_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(C_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(is_aniso,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(is_formatted,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(px,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(py,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(pz,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

     	!======================================================================
	! determine collocation points (knots)
	!======================================================================
	
	if (lpd==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
	elseif (lpd==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
	elseif (lpd==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
	elseif (lpd==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
	elseif (lpd==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
	elseif (lpd==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
	endif

	!======================================================================
	! make boxfile containing the parameters for parallel computing
	!======================================================================

	if (my_rank==0) then

		write(*,*) 'writing boxfile'
	
	endif

	allocate(min_index(1:px*py*pz,1:3),STAT=status)		! minimum element indeces
	allocate(max_index(1:px*py*pz,1:3),STAT=status)		! maximum element indeces
	
	dz=(z_max-z_min)/nz					! width of the elements in z direction
	dy=(y_max-y_min)/ny					! width of the elements in y direction
	dx=(x_max-x_min)/nx					! width of the elements in x direction
	
	bnx=floor(real((nx+1)/px))				! number of elements per processor in x direction
	bny=floor(real((ny+1)/py))				! number of elements per processor in y direction
	bnz=floor(real((nz+1)/pz))				! number of elements per processor in z direction

	if (my_rank==0) then
	
		open(unit=11,file='boxfile',action='WRITE')
		
		write(11,*) "- format description -"
		write(11,*) "total number of processes"
		write(11,*) "number of processes in x direction"
		write(11,*) "number of processes in y direction"
		write(11,*) "number of processes in z direction"
		write(11,*) "single index (npz-1)*py*px+(npy-1)*px+npx"
		write(11,*) "multi index (npx, npy, npz)"
		write(11,*) "index boundaries x (x_min, x_max)"
		write(11,*) "index boundaries y (y_min, y_max)"
		write(11,*) "index boundaries z (z_min, z_max)"
		write(11,*) "physical boundaries x"
		write(11,*) "physical boundaries y"
		write(11,*) "phyiscal boundaries z"
	
		write(11,*) '-------------------'
		write(11,*) px*py*pz
		write(11,*) px
		write(11,*) py
		write(11,*) pz
		write(11,*) '-------------------'
		
	endif
	
	do k=1,pz,1
		do j=1,py,1
			do i=1,px,1
				
				! indices (multiple and single) of the processor boxes (mi=my_rank+1)
				mi=(k-1)*py*px+(j-1)*px+i	
		
				if (my_rank==0) then
				
					write(11,*) mi
					write(11,*) i,j,k
					
				endif
				
				! index boundaries of the processor boxes
				min_index(mi,1)=(i-1)*bnx
				min_index(mi,2)=(j-1)*bny
				min_index(mi,3)=(k-1)*bnz
				
				if (i/=px) then
					max_index(mi,1)=i*bnx
				else
					max_index(mi,1)=nx
				endif
				
				if (j/=py) then
					max_index(mi,2)=j*bny
				else
					max_index(mi,2)=ny
				endif
				
				if (k/=pz) then
					max_index(mi,3)=k*bnz
				else
					max_index(mi,3)=nz
				endif
				
				if (my_rank==0) then
				
					write(11,*) min_index(mi,1), max_index(mi,1)
					write(11,*) min_index(mi,2), max_index(mi,2)
					write(11,*) min_index(mi,3), max_index(mi,3)
				
					! physical boundaries of the processor boxes
					write(11,*) x_min+min_index(mi,1)*dx, x_min+max_index(mi,1)*dx
					write(11,*) y_min+min_index(mi,2)*dy, y_min+max_index(mi,2)*dy
					write(11,*) z_min+min_index(mi,3)*dz, z_min+max_index(mi,3)*dz
					write(11,*) '-------------------'
					
				endif
			enddo
		enddo
	enddo
				
	if (my_rank==0) then
	
		close(UNIT=11)
	
	endif

        !======================================================================
        ! make coordinates
        !======================================================================

        nx_loc=max_index(my_rank+1,1)-min_index(my_rank+1,1)
        ny_loc=max_index(my_rank+1,2)-min_index(my_rank+1,2)
        nz_loc=max_index(my_rank+1,3)-min_index(my_rank+1,3)

        x_min_loc=x_min+min_index(my_rank+1,1)*dx
        x_max_loc=x_min+max_index(my_rank+1,1)*dx

        y_min_loc=y_min+min_index(my_rank+1,2)*dy
        y_max_loc=y_min+max_index(my_rank+1,2)*dy

        z_min_loc=z_min+min_index(my_rank+1,3)*dz
        z_max_loc=z_min+max_index(my_rank+1,3)*dz

        write(99,*) '- indices ----------------------'
        write(99,*) nx_loc, ny_loc, nz_loc
        write(99,*) '- phys. model dimensions -------'
        write(99,*) x_min_loc, x_max_loc
        write(99,*) y_min_loc, y_max_loc
        write(99,*) z_min_loc, z_max_loc

        allocate(x(0:nx_loc,0:lpd))
        allocate(y(0:ny_loc,0:lpd))
        allocate(z(0:nz_loc,0:lpd))

        do i=0,nx_loc
		do n=0,lpd
			x(i,n)=x_min_loc+i*dx+0.5*(1+knots(n))*dx
		enddo
	enddo

	do j=0,ny_loc
		do n=0,lpd
			y(j,n)=y_min_loc+j*dy+0.5*(1+knots(n))*dy
		enddo
	enddo
	
	do k=0,nz_loc
		do n=0,lpd
			z(k,n)=z_max_loc-k*dz-0.5*(1+knots(n))*dz
		enddo
	enddo
 
        do i=0,nz_loc
           write(99,*) z(i,:)
        enddo

	!======================================================================
	! allocate memory
	!======================================================================

	allocate(rhoinv(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(mu(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(lambda(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(A(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(B(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(C(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))

		!=============================================================
		! homogeneous model
		!=============================================================

		if (model_type==1) then
			
                        if (my_rank==0) then

                           write(*,*) 'Homogeneous model'

                        endif

			rhoinv(:,:,:,:,:,:)=1.0/rho_homogen
			mu(:,:,:,:,:,:)=mu_homogen
			lambda(:,:,:,:,:,:)=lambda_homogen
			A(:,:,:,:,:,:)=A_homogen
			B(:,:,:,:,:,:)=B_homogen
			C(:,:,:,:,:,:)=C_homogen
			
		endif
		
		!=============================================================
		! PREM isotropic (without ocean and crust)
		!=============================================================
		
		if (model_type==2) then

                    if (my_rank==0) then

                       write(*,*) 'PREM without crust and ocean'

                    endif

                    !- allocate memory

                    allocate(cp1d(0:nz_loc,0:lpd))
                    allocate(cs1d(0:nz_loc,0:lpd))
                    allocate(rho1d(0:nz_loc,0:lpd))
                    allocate(r(0:nz_loc,0:lpd))
                    
                    !- make 1D model

                    r=z/6371e3

                    do i=0,nz_loc
                       do k=0,lpd

                          !- LID

                          if ((z(i,k)<=6371000) .and. (z(i,k)>=6291000)) then
                             
                             rho1d(i,k)=2.6910+0.6924*r(i,k)
                             cp1d(i,k)=4.1875+3.9382*r(i,k)
                             cs1d(i,k)=2.1519+2.3481*r(i,k)

                          !- LVZ

                          elseif((z(i,k)<=6291000) .and. (z(i,k)>=6151000)) then

                             rho1d(i,k)=2.6910+0.6924*r(i,k)
                             cp1d(i,k)=4.1875+3.9382*r(i,k)
                             cs1d(i,k)=2.1519+2.3481*r(i,k)

                          !- Transition zone

                          elseif((z(i,k)<=6151000) .and. (z(i,k)>=5971000)) then

                             rho1d(i,k)=7.1089-3.8045*r(i,k)
                             cp1d(i,k)=20.3926-12.2569*r(i,k)
                             cs1d(i,k)=8.9496-4.4597*r(i,k)

                          elseif((z(i,k)<=5971000) .and. (z(i,k)>=5771000)) then

                             rho1d(i,k)=11.2494-8.0298*r(i,k)
                             cp1d(i,k)=39.7027-32.6166*r(i,k)
                             cs1d(i,k)=22.3512-18.5856*r(i,k)

                          elseif((z(i,k)<=5771000) .and. (z(i,k)>=5701000)) then

                             rho1d(i,k)=5.3197-1.4836*r(i,k)
                             cp1d(i,k)=19.0957-9.8672*r(i,k)
                             cs1d(i,k)=9.9839-4.9324*r(i,k)

                          !- Lower mantle

                          elseif((z(i,k)<=5701000) .and. (z(i,k)>=5600000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=29.2766-23.6026*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=22.3459-17.2473*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=5600000) .and. (z(i,k)>=3630000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=24.9520-40.4673*r(i,k)+51.4832*r(i,k)*r(i,k)-26.6419*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=11.1671-13.7818*r(i,k)+17.4575*r(i,k)*r(i,k)-9.2777*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=3630000) .and. (z(i,k)>=3480000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=15.3891-5.3181*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=6.9254+1.4672*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          !- Outer core

                          elseif((z(i,k)<=3480000) .and. (z(i,k)>=1221500)) then

                             rho1d(i,k)=12.5815-1.2638*r(i,k)-3.6426*r(i,k)*r(i,k)-5.5281*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=11.0487-4.0362*r(i,k)+4.8023*r(i,k)*r(i,k)-13.5732*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=0.0

                          !- Inner core

                          elseif(z(i,k)<=1221500) then

                             rho1d(i,k)=13.0885-8.8381*r(i,k)*r(i,k)
                             cp1d(i,k)=11.2622-6.3640*r(i,k)*r(i,k)
                             cs1d(i,k)=3.6678-4.4475*r(i,k)*r(i,k)

                          endif

                       enddo

                    enddo

                    rho1d=1000.0*rho1d
                    cp1d=1000.0*cp1d
                    cs1d=1000.0*cs1d

                    !- translate to 3D arrays

                    do i=0,nz_loc
                       do k=0,lpd

                          rhoinv(:,:,i,:,:,k)=1/rho1d(i,k)
                          mu(:,:,i,:,:,k)=cs1d(i,k)*cs1d(i,k)*rho1d(i,k)
                          lambda(:,:,i,:,:,k)=(cp1d(i,k)*cp1d(i,k)-2*cs1d(i,k)*cs1d(i,k))*rho1d(i,k)

                       enddo
                    enddo

                    !- clean up

                    deallocate(cp1d)
                    deallocate(cs1d)
                    deallocate(rho1d)
                    deallocate(r)
                    
                endif
		
		!=============================================================
		! PREM isotropic (without ocean but with crust)
		!=============================================================
		
		if (model_type==3) then

                    if (my_rank==0) then

                       write(*,*) 'PREM without ocean'

                    endif

                    !- allocate memory

                    allocate(cp1d(0:nz_loc,0:lpd))
                    allocate(cs1d(0:nz_loc,0:lpd))
                    allocate(rho1d(0:nz_loc,0:lpd))
                    allocate(r(0:nz_loc,0:lpd))
                    
                    !- make 1D model

                    r=z/6371e3

                    do i=0,nz_loc
                       do k=0,lpd

		       	  if ((z(i,k)<=6371000) .and. (z(i,k)>=6356000)) then
				  
			     rho1d(i,k)=2.60
			     cp1d(i,k)=5.80
			     cs1d(i,k)=3.20
			     
			  elseif ((z(i,k)<=6356000) .and. (z(i,k)>=6346600)) then
				  
			     rho1d(i,k)=2.90
			     cp1d(i,k)=6.80
			     cs1d(i,k)=3.90
		       
                          !- LID

                          elseif ((z(i,k)<=6346600) .and. (z(i,k)>=6291000)) then
                             
                             rho1d(i,k)=2.6910+0.6924*r(i,k)
                             cp1d(i,k)=4.1875+3.9382*r(i,k)
                             cs1d(i,k)=2.1519+2.3481*r(i,k)

                          !- LVZ

                          elseif((z(i,k)<=6291000) .and. (z(i,k)>=6151000)) then

                             rho1d(i,k)=2.6910+0.6924*r(i,k)
                             cp1d(i,k)=4.1875+3.9382*r(i,k)
                             cs1d(i,k)=2.1519+2.3481*r(i,k)

                          !- Transition zone

                          elseif((z(i,k)<=6151000) .and. (z(i,k)>=5971000)) then

                             rho1d(i,k)=7.1089-3.8045*r(i,k)
                             cp1d(i,k)=20.3926-12.2569*r(i,k)
                             cs1d(i,k)=8.9496-4.4597*r(i,k)

                          elseif((z(i,k)<=5971000) .and. (z(i,k)>=5771000)) then

                             rho1d(i,k)=11.2494-8.0298*r(i,k)
                             cp1d(i,k)=39.7027-32.6166*r(i,k)
                             cs1d(i,k)=22.3512-18.5856*r(i,k)

                          elseif((z(i,k)<=5771000) .and. (z(i,k)>=5701000)) then

                             rho1d(i,k)=5.3197-1.4836*r(i,k)
                             cp1d(i,k)=19.0957-9.8672*r(i,k)
                             cs1d(i,k)=9.9839-4.9324*r(i,k)

                          !- Lower mantle

                          elseif((z(i,k)<=5701000) .and. (z(i,k)>=5600000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=29.2766-23.6026*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=22.3459-17.2473*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=5600000) .and. (z(i,k)>=3630000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=24.9520-40.4673*r(i,k)+51.4832*r(i,k)*r(i,k)-26.6419*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=11.1671-13.7818*r(i,k)+17.4575*r(i,k)*r(i,k)-9.2777*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=3630000) .and. (z(i,k)>=3480000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=15.3891-5.3181*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=6.9254+1.4672*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          !- Outer core

                          elseif((z(i,k)<=3480000) .and. (z(i,k)>=1221500)) then

                             rho1d(i,k)=12.5815-1.2638*r(i,k)-3.6426*r(i,k)*r(i,k)-5.5281*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=11.0487-4.0362*r(i,k)+4.8023*r(i,k)*r(i,k)-13.5732*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=0.0

                          !- Inner core

                          elseif(z(i,k)<=1221500) then

                             rho1d(i,k)=13.0885-8.8381*r(i,k)*r(i,k)
                             cp1d(i,k)=11.2622-6.3640*r(i,k)*r(i,k)
                             cs1d(i,k)=3.6678-4.4475*r(i,k)*r(i,k)

                          endif

                       enddo

                    enddo

                    rho1d=1000.0*rho1d
                    cp1d=1000.0*cp1d
                    cs1d=1000.0*cs1d

                    !- translate to 3D arrays

                    do i=0,nz_loc
                       do k=0,lpd

                          rhoinv(:,:,i,:,:,k)=1/rho1d(i,k)
                          mu(:,:,i,:,:,k)=cs1d(i,k)*cs1d(i,k)*rho1d(i,k)
                          lambda(:,:,i,:,:,k)=(cp1d(i,k)*cp1d(i,k)-2*cs1d(i,k)*cs1d(i,k))*rho1d(i,k)

                       enddo
                    enddo

                    !- clean up

                    deallocate(cp1d)
                    deallocate(cs1d)
                    deallocate(rho1d)
                    deallocate(r)
                    
                endif

		!=============================================================
		! PREM isotropic with homogenized crust (80 km)
		!=============================================================
		
		if (model_type==4) then

                    if (my_rank==0) then

                       write(*,*) 'PREM with homogenized crust'

                    endif

                    !- allocate memory

                    allocate(cp1d(0:nz_loc,0:lpd))
                    allocate(cs1d(0:nz_loc,0:lpd))
                    allocate(rho1d(0:nz_loc,0:lpd))
                    allocate(r(0:nz_loc,0:lpd))
                    
                    !- make 1D model

                    r=z/6371e3

                    do i=0,nz_loc
                       do k=0,lpd

		       	  if ((z(i,k)<=6371000) .and. (z(i,k)>=6291000)) then
				  
			     rho1d(i,k)=3.1207
			     cp1d(i,k)=7.3584
			     cs1d(i,k)=4.0580
			     
                          elseif((z(i,k)<=6291000) .and. (z(i,k)>=6151000)) then

                             rho1d(i,k)=2.6910+0.6924*r(i,k)
                             cp1d(i,k)=4.1875+3.9382*r(i,k)
                             cs1d(i,k)=2.1519+2.3481*r(i,k)

                          !- Transition zone

                          elseif((z(i,k)<=6151000) .and. (z(i,k)>=5971000)) then

                             rho1d(i,k)=7.1089-3.8045*r(i,k)
                             cp1d(i,k)=20.3926-12.2569*r(i,k)
                             cs1d(i,k)=8.9496-4.4597*r(i,k)

                          elseif((z(i,k)<=5971000) .and. (z(i,k)>=5771000)) then

                             rho1d(i,k)=11.2494-8.0298*r(i,k)
                             cp1d(i,k)=39.7027-32.6166*r(i,k)
                             cs1d(i,k)=22.3512-18.5856*r(i,k)

                          elseif((z(i,k)<=5771000) .and. (z(i,k)>=5701000)) then

                             rho1d(i,k)=5.3197-1.4836*r(i,k)
                             cp1d(i,k)=19.0957-9.8672*r(i,k)
                             cs1d(i,k)=9.9839-4.9324*r(i,k)

                          !- Lower mantle

                          elseif((z(i,k)<=5701000) .and. (z(i,k)>=5600000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=29.2766-23.6026*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=22.3459-17.2473*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=5600000) .and. (z(i,k)>=3630000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=24.9520-40.4673*r(i,k)+51.4832*r(i,k)*r(i,k)-26.6419*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=11.1671-13.7818*r(i,k)+17.4575*r(i,k)*r(i,k)-9.2777*r(i,k)*r(i,k)*r(i,k)

                          elseif((z(i,k)<=3630000) .and. (z(i,k)>=3480000)) then

                             rho1d(i,k)=7.9565-6.4761*r(i,k)+5.5283*r(i,k)*r(i,k)-3.0807*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=15.3891-5.3181*r(i,k)+5.5242*r(i,k)*r(i,k)-2.5514*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=6.9254+1.4672*r(i,k)-2.0834*r(i,k)*r(i,k)+0.9783*r(i,k)*r(i,k)*r(i,k)

                          !- Outer core

                          elseif((z(i,k)<=3480000) .and. (z(i,k)>=1221500)) then

                             rho1d(i,k)=12.5815-1.2638*r(i,k)-3.6426*r(i,k)*r(i,k)-5.5281*r(i,k)*r(i,k)*r(i,k)
                             cp1d(i,k)=11.0487-4.0362*r(i,k)+4.8023*r(i,k)*r(i,k)-13.5732*r(i,k)*r(i,k)*r(i,k)
                             cs1d(i,k)=0.0

                          !- Inner core

                          elseif(z(i,k)<=1221500) then

                             rho1d(i,k)=13.0885-8.8381*r(i,k)*r(i,k)
                             cp1d(i,k)=11.2622-6.3640*r(i,k)*r(i,k)
                             cs1d(i,k)=3.6678-4.4475*r(i,k)*r(i,k)

                          endif

                       enddo

                    enddo

                    rho1d=1000.0*rho1d
                    cp1d=1000.0*cp1d
                    cs1d=1000.0*cs1d

                    !- translate to 3D arrays

                    do i=0,nz_loc
                       do k=0,lpd

                          rhoinv(:,:,i,:,:,k)=1/rho1d(i,k)
                          mu(:,:,i,:,:,k)=cs1d(i,k)*cs1d(i,k)*rho1d(i,k)
                          lambda(:,:,i,:,:,k)=(cp1d(i,k)*cp1d(i,k)-2*cs1d(i,k)*cs1d(i,k))*rho1d(i,k)

                       enddo
                    enddo

                    !- clean up

                    deallocate(cp1d)
                    deallocate(cs1d)
                    deallocate(rho1d)
                    deallocate(r)
                    
                endif
	
						
		!=============================================================
		! AK135
		!=============================================================
		
		if (model_type==7) then

                    if (my_rank==0) then

                       write(*,*) 'AK135'

                    endif

                    !- allocate memory

                    allocate(cp1d(0:nz_loc,0:lpd))
                    allocate(cs1d(0:nz_loc,0:lpd))
                    allocate(rho1d(0:nz_loc,0:lpd))
                    allocate(r(0:nz_loc,0:lpd))
                    
		    !- fill arrays for ak135
	    
		    ak135_r(:)=(/6371.0,6351.0,6351.0,6336.0,6336.0,6293.5,6251.0,6251.0,6206.0,6161.0,&
                                 6161.0,6111.0,6061.0,6011.0,5961.0,5961.0,5911.0,&
	    			 5861.0,5811.0,5761.0,5711.0,5711.0,5661.0,5611.0,5561.5,5512.0,5462.5,&
                                 5413.0,5363.5,5314.0,5264.5,5215.0,5165.5,&
				 5116.0,5066.5,5017.0,4967.5,4918.0,4868.5,4819.0,4769.5,4720.0,4670.5,4621.0,4571.5,4522.0,4472.5,&
				 4423.0,4373.5,4324.0,4274.5,4225.0,4175.5,4126.0,4076.5,4027.0,3977.5,3928.0,3878.5,3829.0,3779.5,&
				 3731.0,3681.0,3631.0,3631.0,3581.3,3531.7,3479.5,3479.5,3431.6,3381.3,3331.0,3280.6,3230.3,&
				 3180.0,3129.7,3079.4,3029.0,2978.7,2928.4,2878.3,2827.7,2777.4,2727.0,2676.7,2626.4,&
				 2576.0,2525.7,2475.4,2425.0,2374.7,2324.4,2274.1,2223.7,2173.4,2123.1,2072.7,2022.4,&
				 1972.1,1921.7,1871.4,1821.1,1770.7,1720.4,1670.1,1619.8,1569.4,1519.1,1468.8,1418.4,&
				 1368.1,1317.8,1267.4,1217.5,1217.5,1166.4,1115.7,1064.9,1014.3,963.5,912.83,862.11,&
				 811.4,760.69,709.98,659.26,608.55,557.84,507.13,456.41,405.7,354.99,304.28,253.56,202.85,152.14,&
				 101.43,50.71,0.0/)
				 
		   ak135_r=1000*ak135_r

		   ak135_cp(:)=(/5.800000,5.800000,6.500000,6.500000,8.040000,8.045000,8.050000,8.050000,8.175000,8.300700,&
				 8.300700,8.482200,8.665000,8.847600,9.030200,9.360100,9.528000,9.696200,9.864000,10.032000,&
				 10.200000,10.790900,10.922200,11.055300,11.135500,11.222800,11.306800,11.389700,11.470400,11.549300,&
				 11.626500,11.702000,11.776800,11.849100,11.920800,11.989100,12.057100,12.124700,12.191200,12.255800,&
				 12.318100,12.381300,12.442700,12.503000,12.563800,12.622600,12.680700,12.738400,12.795600,12.852400,&
				 12.909300,12.966300,13.022600,13.078600,13.133700,13.189500,13.246500,13.301700,13.358400,13.415600,&
				 13.474100,13.531100,13.589900,13.649800,13.649800,13.653300,13.657000,13.660100,8.000000,8.038200,&
				 8.128300,8.221300,8.312200,8.400100,8.486100,8.569200,8.649600,8.728300,8.803600,8.876100,&
				 8.946100,9.013800,9.079200,9.142600,9.204200,9.263400,9.320500,9.376000,9.429700,9.481400,&
				 9.530600,9.577700,9.623200,9.667300,9.710000,9.751300,9.791400,9.830400,9.868200,9.905100,&
				 9.941000,9.976100,10.010300,10.043900,10.076800,10.109500,10.141500,10.173900,10.204900,10.232900,&
				 10.256500,10.274500,10.285400,10.289000,11.042700,11.058500,11.071800,11.085000,11.098300,11.116600,&
				 11.131600,11.145700,11.159000,11.171500,11.183200,11.194100,11.204100,11.213400,11.221900,11.229500,&
				 11.236400,11.242400,11.247700,11.252100,11.255700,11.258600,11.260600,11.261800,11.262200/)

                   ak135_cp=1000*ak135_cp
				 
		   ak135_cs(:)=(/3.460000,3.460000,3.850000,3.850000,4.480000,4.490000,4.500000,4.500000,4.509000,4.518400,&
			 	 4.518400,4.609400,4.696400,4.783200,4.870200,5.080600,5.186400,5.292200,5.398900,5.504700,&
				 5.610400,5.960700,6.089800,6.210000,6.242400,6.279900,6.316400,6.351900,6.386000,6.418200,&
				 6.451400,6.482200,6.513100,6.543100,6.572800,6.600900,6.628500,6.655400,6.681300,6.707000,&
				 6.732300,6.757900,6.782000,6.805600,6.828900,6.851700,6.874300,6.897200,6.919400,6.941600,&
				 6.962500,6.985200,7.006900,7.028600,7.050400,7.072200,7.093200,7.114400,7.136800,7.158400,&
				 7.180400,7.203100,7.225300,7.248500,7.248500,7.259300,7.270000,7.281700,0.000000,0.000000,&
				 0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
				 0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
				 0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
				 0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
				 0.000000,0.000000,0.000000,0.000000,3.504300,3.518700,3.531400,3.543500,3.555100,3.566100,&
                                 3.576500,3.586400,3.595700,3.604400,3.612600,3.620200,3.627200,3.633700,3.639600,3.645000,&
                                 3.649800,3.654000,3.657700,3.660800,3.663300,3.665300,3.666700,3.667500,3.667800/)
                                 
                 ak135_cs=1000*ak135_cs

		 ak135_rho(:)=(/2.720000,2.720000,2.920000,2.920000,3.320000,3.345000,3.371000,3.371100,3.371100,3.324300,&
				3.324300,3.366300,3.411000,3.457700,3.506800,3.931700,3.927300,3.923300,3.921800,3.920600,&
				3.920100,4.238700,4.298600,4.356500,4.411800,4.465000,4.516200,4.565400,4.592600,4.619800,&
				4.646700,4.673500,4.700100,4.726600,4.752800,4.779000,4.805000,4.830700,4.856200,4.881700,&
				4.906900,4.932100,4.957000,4.981700,5.006200,5.030600,5.054800,5.078900,5.102700,5.126400,&
				5.149900,5.173200,5.196300,5.219200,5.242000,5.264600,5.287000,5.309200,5.331300,5.353100,&
				5.374800,5.396200,5.417600,5.438700,5.693400,5.719600,5.745800,5.772100,9.914500,9.994200,&
				10.072200,10.148500,10.223300,10.296400,10.367900,10.437800,10.506200,10.573100,10.638500,10.702300,&
				10.764700,10.825700,10.885200,10.943400,11.000100,11.055500,11.109500,11.162300,11.213700,11.263900,&
				11.312700,11.360400,11.406900,11.452100,11.496200,11.539100,11.580900,11.621600,11.661200,11.699800,&
				11.737300,11.773700,11.809200,11.843700,11.877200,11.909800,11.941400,11.972200,12.000100,12.031100,&
				12.059300,12.086700,12.113300,12.139100,12.703700,12.728900,12.753000,12.776000,12.798000,12.818800,&
				12.838700,12.857400,12.875100,12.891700,12.907200,12.921700,12.935100,12.947400,12.958600,12.968800,&
				12.977900,12.985900,12.992900,12.998800,13.003600,13.007400,13.010000,13.011700,13.012200/)

                  ak135_rho=1000*ak135_rho

			do i=0,nz_loc
				do k=0,lpd
				
					do n=1,138
			
						if ((z(i,k)>=ak135_r(n+1)) .and. (z(i,k)<=ak135_r(n))) then
			
                                                       if ((ak135_r(n)-ak135_r(n+1))==0) then

                                                                cp1d(i,k)=ak135_cp(n)
                                                                cs1d(i,k)=ak135_cs(n)
                                                                rho1d(i,k)=ak135_rho(n)

                                                        else
                        
        							ratio=(ak135_r(n)-z(i,k))/(ak135_r(n)-ak135_r(n+1))
					
	        						cp1d(i,k)=ak135_cp(n)*(1-ratio)+ak135_cp(n+1)*ratio
		        					cs1d(i,k)=ak135_cs(n)*(1-ratio)+ak135_cs(n+1)*ratio
			        				rho1d(i,k)=ak135_rho(n)*(1-ratio)+ak135_rho(n+1)*ratio

                                                        endif
						
						endif
				
					enddo
			
				enddo
			enddo
						
			
      

                    !- translate to 3D arrays

                    do i=0,nz_loc
                       do k=0,lpd

                          rhoinv(:,:,i,:,:,k)=1/rho1d(i,k)
                          mu(:,:,i,:,:,k)=cs1d(i,k)*cs1d(i,k)*rho1d(i,k)
                          lambda(:,:,i,:,:,k)=(cp1d(i,k)*cp1d(i,k)-2*cs1d(i,k)*cs1d(i,k))*rho1d(i,k)

                       enddo
                    enddo

                    !- clean up

                    deallocate(cp1d)
                    deallocate(cs1d)
                    deallocate(rho1d)
                    deallocate(r)
                    
							
                endif


		
		!=============================================================
		! SPICE Tottori benchmark
		!=============================================================
		
		if (model_type==200) then

                    if (my_rank==0) then

                       write(*,*) 'SPICE Tottori '

                    endif

        !- allocate memory

        allocate(cp1d(0:nz_loc,0:lpd))
        allocate(cs1d(0:nz_loc,0:lpd))
        allocate(rho1d(0:nz_loc,0:lpd))
        allocate(r(0:nz_loc,0:lpd))

        !- make 1D model

        r=z/6371e3

        do i=0,nz_loc
                do k=0,lpd

                        !- crust
                
                        if ((z(i,k)<=6371000) .and. (z(i,k)>=6369000)) then
                                
                             rho1d(i,k)=2.60
                             cp1d(i,k)=5.50
                             cs1d(i,k)=3.18
                        
                        elseif ((z(i,k)<=6369000) .and. (z(i,k)>=6355000)) then
                                
                             rho1d(i,k)=2.70
                             cp1d(i,k)=6.05
                             cs1d(i,k)=3.50
                
                        elseif ((z(i,k)<=6355000) .and. (z(i,k)>=6333000)) then

                             rho1d(i,k)=2.8
                             cp1d(i,k)=6.6
                             cs1d(i,k)=3.81

                        elseif(z(i,k)<=6333000) then

                             rho1d(i,k)=3.1
                             cp1d(i,k)=8.03
                             cs1d(i,k)=4.62

                        endif

                enddo

            enddo

            rho1d=1000.0*rho1d
            cp1d=1000.0*cp1d
            cs1d=1000.0*cs1d

           !- translate to 3D arrays

            do i=0,nz_loc
               do k=0,lpd

                  rhoinv(:,:,i,:,:,k)=1/rho1d(i,k)
                  mu(:,:,i,:,:,k)=cs1d(i,k)*cs1d(i,k)*rho1d(i,k)
                  lambda(:,:,i,:,:,k)=(cp1d(i,k)*cp1d(i,k)-2*cs1d(i,k)*cs1d(i,k))*rho1d(i,k)

               enddo
            enddo

                    !- clean up

            deallocate(cp1d)
            deallocate(cs1d)
            deallocate(rho1d)
            deallocate(r)
                    
                endif


		
	!======================================================================
	! save to files
	!======================================================================

		!==============================================================
		! open files, formatted or unformatted
		!==============================================================

		call int2str(my_rank,junk)
			
		OPEN(UNIT=11,FILE='rhoinv'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		OPEN(UNIT=12,FILE='mu'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		OPEN(UNIT=13,FILE='lambda'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		
		if (is_aniso==1) then
				
			OPEN(UNIT=14,FILE='A'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
			OPEN(UNIT=15,FILE='B'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
			OPEN(UNIT=16,FILE='C'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
				
		endif
		
		!==============================================================
		! write parameters to files, formatted or unformatted
		!==============================================================
			
		write(11) rhoinv(:,:,:,:,:,:)
		write(12) mu(:,:,:,:,:,:)
		write(13) lambda(:,:,:,:,:,:)
			
		if (is_aniso==1) then
				
			write(14) A(:,:,:,:,:,:)
			write(15) B(:,:,:,:,:,:)
			write(16) C(:,:,:,:,:,:)
				
		endif
				
		close(UNIT=11)
		close(UNIT=12)
		close(UNIT=13)
		
		if (is_aniso==1) then
		
			close(UNIT=14)
			close(UNIT=15)
			close(UNIT=16)
			
		endif

!==============================================================================
! clean up
!==============================================================================
	
		deallocate(rhoinv)
		deallocate(mu)
		deallocate(lambda)
		
		deallocate(A)
		deallocate(B)
		deallocate(C)
	
	deallocate(min_index)
	deallocate(max_index)

        deallocate(x)
        deallocate(y)
        deallocate(z)

        close(unit=99)
	
	call MPI_Finalize(ierr)

end program generate_models



!==============================================================================
! subroutine for integer to string conversion
!==============================================================================

subroutine int2str(value, string)
implicit none

	integer, intent(in) :: value
	character(len=*), intent(inout) :: string
	
	character(len=10) :: c
	integer :: k, n, new_value, is
	real :: e
	
	
	e=1e9
	is=0
	
	if (value==0) then
		string(1:1)='0'
		string(2:10)=' '
	else
	
		new_value=value
	
		do k=1,10
			c(k:k)=char(floor(new_value/e)+48)
		
			if ((floor(new_value/e)==0) .and. (is==0)) then
				n=k
			else
				is=1
			endif
		
			!write(*,*) c
			new_value=new_value-e*floor(new_value/e)
			e=e/10
			string(k:k)=' '
		
		enddo
		
		string(1:10-n)=c(n+1:10)
		
	endif

	if (len(string)>10) then
		string(11:len(string))=' '
	endif

return
end subroutine int2str
